/*
 * Copyright 2003, Research Engineering Development.
 * Author: Alfred Perlstein <alfred@freebsd.org>
 */

#include <stdio.h>
#include <stdlib.h>
#include <err.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#ifdef __unused
#define UNUSED(arg)	arg __unused
#else
#define UNUSED(arg)	arg
#endif

int
main(UNUSED(int argc), char **argv)
{
	pid_t gotpid, childpid;
	/* char **ap; */
	int status;
	int ch;

	if (argv[1] == NULL)
		errx(1, "usage: %s <program> [args...]", argv[0]);

	fprintf(stderr,
	    "Replay (c) 2003 Research Engineering Development\n");

rerun:
	childpid = vfork();
	switch (childpid) {
	case 0:
		execvp(argv[1], argv + 1);
		err(1, "evecvp %s", argv[1]);
	case -1:
		err(1, "fork");
	}
	gotpid = waitpid(childpid, &status, 0);
	if (WIFEXITED(status)) {
		if (WEXITSTATUS(status) == EXIT_SUCCESS)
			exit(EXIT_SUCCESS);
	}

inputagain:
	fprintf(stderr, "program: %s ", argv[1]);
#if 0
	for (ap = argv + 1; *ap != NULL; ap++)
		fprintf(stderr, "%s%s", ap[0], ap[1] == NULL ? "\n" : " ");
#endif
	if (WIFEXITED(status))
		fprintf(stderr, "exited with code %d.\n", WIFEXITED(status));
	else if (WIFSIGNALED(status))
		fprintf(stderr, "exited due to signal %d%s.\n",
		    WTERMSIG(status), WCOREDUMP(status) ? " core dumped" : "");
	fprintf(stderr, "run again? [y/n]\n");
	fpurge(stdin);
	ch = getchar();
	switch (ch) {
	case 'y':
		goto rerun;
	case 'n':
	case EOF:
		exit(EXIT_FAILURE);
	default:
		fprintf(stderr,
		    "invalid choice '%c' please choose 'y' or 'n'\n", ch);
		goto inputagain;
	}
}
